PROGRAM bondenforce
!
!*******************************************************************************
! Solve FH incomplete bond economy using value function iteration
!
! Incomplete market with state-un
! contigent bond, no adjustment cost
! 
! Utility function: u(c)=(c^(1-sigma)-1)/(1-sigma)
! No aggregate uncertainty
! productivity shock
! Related code: val.m
! Problem: V(A,b,k) = max (1-beta)(u(c))+beta* sum (p(A'|A)*v(A',b',k')
!                     s.t. c = A * k^alpha + (1-delta) * k + R * b - b' - k'
!                          b' >= B(a,k')
! where, B(a,k') is the endogenous borrowing constraints from limited enforcement.
!
! Author: Bai and Jing
! March 20, 2009
!******************************************************************************

USE bondenforcevar

IMPLICIT NONE

INCLUDE 'mpif.h'

integer                              :: i,j,m,ii
integer, dimension(la,lb,lk)         :: setib,setik
real*8, dimension(la,lb,lk)          :: v0,setkp,setbp, setc

integer, dimension(la*lb*lk)         :: setibp,setikp
real*8, dimension(la*lb*lk)          :: den

real*8                                         :: sav,gdp,deb,dratio
! Run-time measure variables
character(8)                         :: tdate
character(10)                        :: ttime
real*8                               :: runtime1, runtime2, runtime3, runtime4,totaltime,es

! Input to the model
open(unit=1,  file =trim(dirin)//'shock9.dat')
open(unit=3,  file =trim(dirout)//'para.dat')
open(unit=13, file =trim(dirout)//'kgrid.dat')
open(unit=15, file =trim(dirout)//'bgrid.dat')
open(unit=17, file =trim(dirout)//'autk.dat')

! Output to the model
open(unit=4,  file =trim(dirout)//'shock.dat')
open(unit=5,  file =trim(dirout)//'vbondout.txt')
open(unit=16, file =trim(dirout)//'bondV.dat')

open(unit=8,  file =trim(dirout)//'bondKp.dat')
open(unit=9,  file =trim(dirout)//'bondBp.dat')
open(unit=10, file =trim(dirout)//'bondSai.dat')
open(unit=12, file =trim(dirout)//'ibondKp.dat')
open(unit=14, file =trim(dirout)//'ibondBp.dat')
open(unit=20, file =trim(dirout)//'bondC.dat')

open(unit=30, file =trim(dirout)//'autU.dat')
open(unit=32, file =trim(dirout)//'autkp.dat')
open(unit=40, file =trim(dirout)//'bbar.dat')
open(unit=42, file =trim(dirout)//'ibbar.dat')
open(unit=44, file =trim(dirout)//'test.dat')
open(unit=46, file =trim(dirout)//'psk.dat')
open(unit=50, file =trim(dirout)//'tbbar.dat')
open(unit=100, file =trim(dirout)//'tmp.dat')

!*******************************************************************
! Initial MPI process
call MPI_INIT(ierr)

! Get my rank
call MPI_COMM_RANK(MPI_COMM_WORLD, myrank, ierr)
call MPI_COMM_SIZE(MPI_COMM_WORLD, mysize, ierr)

! Get starting time
call date_and_time(tdate,ttime)

!*******************************************************************
! Initial the shocks and state spaces
call initial(v0)

! Allocat works to different processors
do i = 1, np
	istart(i) = (i-1)*mk+1
end do
if (myrank==1) print *, 'Start!'
do i = 1,np
	if (myrank == i-1) then
		inda  = istart(i)
		!print *, 'I am processor ', myrank, ' and I will compute from ', inda
	end if
end do
!*******************************************************************
! Begin solving the problem
! First iterate beta to clear markets at R=1.04
! Second iterate over value function together with endogenous bond limits
! Last iterate over distribution

 call biterR(v0,setib,setik,setkp,setbp,setc,setikp,setibp,es,sav,gdp,deb,dratio,den)

!*******************************************************************
! Writeout results
if (myrank==0) then
	call cpu_time(totaltime)
	write(*,*) '***********************************'
	write(*,*) 'Total running time: ', totaltime
	
	!****************************************************************************
	! Write out all the results
	! Write out parameters used for simulation
	rewind(3)
	write(3,*) rbeta
	write(3,*) sigma
	write(3,*) alpha
	write(3,*) delta
	write(3,*) R
	write(3,*) la
	write(3,*) lk
	write(3,*) lb
	write(3,*) gama
	write(3,*) lamda
	rewind(3)
	
	! Write out the policy rule and value function
	rewind(16)
	rewind(8)
	rewind(9)
	rewind(12)
	rewind(14)
	rewind(20)
	do i = 1,la
		do j = 1,lb
			do m = 1,lk
				write (8,*)   setkp(i,j,m)
				write (9,*)   setbp(i,j,m)
				write (12,*)  setik(i,j,m)
				write (14,*)  setib(i,j,m)
				write (16,*)  v0(i,j,m)
				write (20,*)  setc(i,j,m)
			end do
		end do
	end do
	rewind(8)
	rewind(9)
	rewind(12)
	rewind(14)
	rewind(16)
	rewind(20)
	
	! Write out endogenous borrowing constraints
	rewind(42)
	do i = 1,la
		do j = 1, lk
			write(42,*) ibbar0(i,j)
		end do
	end do
	rewind(42)
	
	rewind(40)
	write(40,*) la
	write(40,*) lk
	do ii = 1, iterB
		do i = 1,la
			do j = 1, lk
				write(40,*) bbar(i,j,ii)
			end do
		end do
	end do
	rewind(40)
	
	! Write out shock process
	rewind(4)
	do i = 1,la
		write(4,*) A(i)
	end do
	do i = 1,la
		do j = 1,la
			write(4,*) p(i,j)
		end do
	end do
	rewind(4)
	
	! Write out invariant distribution
	rewind(10)
	do i = 1, la*lb*lk
		write(10,*) den(i)
	end do
	rewind(10)
	
	rewind(46)
	do i = 1, lk
		do j = 1, lb
			write(46,*) psk0(i,j)
		end do
	end do
	rewind(46)	

	rewind(17)
	do i=1,autlk
		write(17,*) autk(i)
	end do
	rewind(17)
	
	rewind(30)
	do i=1,la
		do j=1,lk
			write (30, *) uu(i,j)			
		end do
	end do
	rewind(30)

	
	! Write summary output
	write (5, *) 'Date:  ', tdate
	write (5, *) 'Time:  ', ttime
	write (5, *) 'Parameters for this run:'
	write (5,12) 'R      =       ', R
	write (5,12) 'beta   =       ', rbeta
	write (5,12) 'sigma  =       ', sigma
	write (5,12) 'delta  =       ', delta
	write (5,12) 'alpha  =       ', alpha
	write (5,12) 'gama   =       ', gama
	write (5,12) 'lamda  =       ', lamda
	write (5, *)
	write (5,12) 'kmin   =       ', kmin
	write (5,12) 'kmax   =       ', kmax
	write (5,12) 'bmin   =       ', bmin
	write (5,12) 'bmax   =       ', bmax
	
	write (5,14) 'la     =       ', la
	write (5,14) 'lk     =       ', lk
	write (5,14) 'lb     =       ', lb
	write (5, *)
	write (5,*) 'excess demand = ', es
	write (5,*)  'saving = ', sav
	call cpu_time(totaltime)
	
	write (5, *) 'Running time: ', totaltime  
	11 format('Excess demand = ', F8.4)
	12 format(1x, A10,2x, F8.4)
	14 format(1x, A10,2x, I5)
	15 format(1x, A10,2x, E8.2)
end if
call MPI_FINALIZE(ierr)
END PROGRAM bondenforce











