subroutine Simu(simua, isimua, simuk0, isimuk0, simub0, isimub0)

! starting points for simulation
!*******************************************************
! Simulation for the bond market:
! Input:
!       Productivity shock process A and P
!       Invariant distribution over states (A,K,B): miu (in CRS format)
!
!       miu(la*lk*lb):        stationary distribution
!
! Output:
!       Following simulation results for NC countries and T period
!       (1) Initial year state (A0, K0, B0)
!       (2) A(t) for t = 2: T-1
!
!       simua(NC,T):                simulated productivity process
!       simuk0(NC,1):               simulated capital stock
!       simub0(NC,1):               simulated bonds holding
!
! Parameters
!       T:                          number of years to be simulated
!       NC:                         number of countries to be simulated
!       la:                         number of productivity levels
!       lk:                         number of capital grid
!       lb:                         number of bond grid
!
!*******************************************************


USE simuvar

implicit none

! Results
integer, intent(out), dimension(T,NC)   :: isimua
integer, intent(out), dimension(1,NC)   :: isimub0, isimuk0
real*8, intent(out), dimension(T,NC)      :: simua
real*8, intent(out), dimension(1,NC)      :: simuk0,simub0
real*8, dimension(T,NC)                   :: tsimua
! Working arrays
integer                    :: i, j, m, s1, tt,jj

integer, dimension(NC)     :: state
real*8, dimension(NC)        :: tran1
real*8, dimension(T-1)       :: tran
real*8, dimension(T-1,NC)    :: ranp

real*8, dimension(la)        :: invp, cuminvp
complex, dimension(la)     :: eval
real*8, dimension(la)        :: reval
real*8, dimension(la,la)     :: tmpm, tmpm1, tmpmm
integer                       :: ttb, ttk, tmpt,tta,za

real*8:: tmiu(lk*lb),ctmiu(lk*lb),dkk,tm(lb),tmpk,ctm(lb),csmu(la*lk*lb),smu(la*lk*lb)
!*******************************************************
! Simulation
select case (itest)
case (0)
	! Step (1)
	! Generate T*nc random numbers
 
	!s1=233156          ! Select 9
	!call rnset(s1)
	!call rnget(s1)
	!print *, 'rseed=', s1
		
	! Draw random number for initial state  according to invariant distribution invp
	call drnun(NC,tran1)
	select case (igroup)
	case (1)
		smu(1:3*lk*lb)=miu(1:3*lk*lb)/sum(miu(1:3*lk*lb))
		za=3
	case (2)
		smu(1:3*lk*lb)=miu(3*lk*lb+1:6*lk*lb)/sum(miu(3*lk*lb+1:6*lk*lb))
		za=3
	case (3)
		smu(1:3*lk*lb)=miu(6*lk*lb+1:9*lk*lb)/sum(miu(6*lk*lb+1:9*lk*lb))
		za=3
	case (4) 
		smu(1:3*lk*lb)=miu(1:3*lk*lb)
		smu(3*lk*lb+1:6*lk*lb)=miu(6*lk*lb+1:9*lk*lb)
		smu(1:6*lk*lb)=smu(1:6*lk*lb)/sum(smu(1:6*lk*lb))
		
		za=6
	case (0)
		smu=miu
		za=la
	end select
	
	if (igroup>0) then
		csmu(1)=smu(1)
		do j=2,za*lk*lb
			csmu(j)=csmu(j-1)+smu(j)
		end do
	else
		csmu=cummiu
	end if
		
	do i=1, NC
		isimua(1,i)  = 1
		do m = 1,za*lk*lb
			if (csmu(m) >= tran1(i)) then
				ia           = ceiling(real(m)/real(lb*lk))
				ib           = ceiling(real(m-(ia-1)*lk*lb)/real(lk))
				ik           = m - (ia-1)*lb*lk - (ib-1)*lk
				if (igroup>0) then
					isimua(1,i)  = ia+(igroup-1)*za
				else
					isimua(1,i)=ia
				end if	
				
				isimub0(1,i) = ib
				isimuk0(1,i) = ik
				simua(1,i)   = A(ia)  ! Initial productivity shock
				simub0(1,i)  = b(ib)  ! Initital bonds is zero!
				simuk0(1,i)  = K(ik)  ! Initial capital 
				exit
			end if
		end do
	end do

	! Random number for the following state a'
	tran = 0.0
	do i=1,nc
		call drnun(T-1,tran)
		ranp(:,i)=tran
	end do

	! Find initial state for each country
	do i=1,nc              
		do j=2,T                       ! Iterate over periods 2 : T
			isimua(j,i)  = 1
			do m = 1,la
				if (cumpai(isimua(j-1,i),m)>=ranp(j-1,i)) then
					isimua(j,i) = m
					simua(j,i)  = a(m)
					exit
				end if
			end do
		end do                         ! end of simulation for T
	end do                             ! end of simulation for nc 
case (1)
	rewind(33)
	do i=1,NC
		do j=1,T
			read(33,*) tsimua(j,i)
			tsimua(j,i) = exp(tsimua(j,i))/50.0
			tt  = minloc(abs(a-tsimua(j,i)),1)
			!simua(j,i)  = a(tt)
			isimua(j,i) = tt
			simua(j,i)=tsimua(j,i)
			
		end do
	end do
	rewind(33)


	! Choice over initial k and b
	select case (iskb)
	case (0) ! start with any point chosen
		ttk          = minloc(abs(k-k(lk)/2),1)
		ttb          = minloc(abs(b-0.0),1)
		isimuk0(1,:) = ttk
		isimub0(1,:) = ttb
		simuk0(1,:) = k(ttk)
		simub0(1,:) = b(ttb)
	case (1) ! start with the most possible point (with highest measure)
		do i=1,NC
			tt = isimua(1,i)
			tmiu = cummiu((tt-1)*lk*lb+1:tt*lk*lb)
			tmpt = maxloc(tmiu,1)
			ib           = ceiling(real(tmpt)/real(lk))
			ik           = tmpt - (ib-1)*lk
			isimuk0(1,i) = ik
			isimub0(1,i) = ib
		
			simuk0(1,i)  = k(ik)
			simub0(1,i)  = b(ib)
		end do
	case (2) ! start randomly
		call drnun(NC,tran1)
		do i=1,NC
			tt=isimua(1,i)
			tmiu = miu((tt-1)*lk*lb+1:tt*lk*lb)
			tmiu = tmiu/sum(tmiu)
			ctmiu(1)=tmiu(1)
			do j=2,lk*lb
				ctmiu(j)=ctmiu(j-1)+tmiu(j)
			end do
			isimub0(1,i)  = 1
			isimuk0(1,i)  = 1
			do m = 2,lk*lb
				if (ctmiu(m) >= tran1(i)) then
					ib           = ceiling(real(m)/real(lk))
					ik           = m - (ib-1)*lk
					isimub0(1,i) = ib
					isimuk0(1,i) = ik
					simub0(1,i)  = b(ib)  ! Initital bonds is zero!
					simuk0(1,i)  = K(ik)  ! Initial capital 
					exit
				end if
			end do
		end do
	case (3) ! k from data,most possible b
		rewind(35)
		do i=1,nc	
			
			read(35,*) dkk
			tmpk=(simua(1,i)*dkk)**(1.0/(1.0-alpha))
			tt  = minloc(abs(k-tmpk),1)
			simuk0(1,i)  = k(tt)
			isimuk0(1,i) = tt
			
			tta=isimua(1,i)
			ttk=isimuk0(1,i)
			
			do jj=1,lb
				tm(j)=miu((tta-1)*lk*lb+(jj-1)*lk+ttk)
			end do
			tm = tm/sum(tm)
			isimub0(1,i)=maxloc(tm,1)
			simub0(1,i)=b(isimub0(1,i))
		end do
	case (4) ! k from data,random draw b
		call drnun(NC,tran1)
		rewind(35)
		do i=1,nc	
			
			read(35,*) dkk
			tmpk=(simua(1,i)*dkk)**(1.0/(1.0-alpha))
			tt  = minloc(abs(k-tmpk),1)
			simuk0(1,i)  = k(tt)
			isimuk0(1,i) = tt
			
			tta=isimua(1,i)
			ttk=isimuk0(1,i)
			do jj=1,lb
				tm(j)=miu((tta-1)*lk*lb+(jj-1)*lk+ttk)
			end do
			tm=tm/sum(tm)
			
			ctm(1)=tm(1)
			do j=2,lb
				ctm(j)=ctm(j-1)+tm(j)
			end do
			isimub0(1,i)  = 1
			do m = 2,lb
				if (ctm(m) >= tran1(i)) then
					ib           = m
					isimub0(1,i) = ib
					simub0(1,i)  = b(ib)  ! Initital bonds is zero!
					exit
				end if
			end do

			
		end do
		
	end select
	
	
	
end select
END subroutine Simu

