Program SimuComplete

!*******************************************************
! Simulate and test for complete market results
!
! Algorithm:
!       First solve the invariant distribution of a (prod shocks)
!       From complete market result, kp only depends on current a, so there are only
!          8 numbers of k will have positive density in the invariant distribution according to each a.
!       We assume that each country start to the k0 according to a0 (i.e. a(-1) = a0)
!       It is hard to calibrate exactly b0. So start with b0=0 for all series. Then simulate
!          for 10000 periods then drop the first 9960 periods. Assume that last 40 periods are in
!          invariant distribution!
!
! Input:
!       Productivity shock process A and P
!       Decision rule: gb, gk
!       State grids  : k,  b
!
!       gb(la*lb,lk) :        bond holding decision bp
!       gk(la*lb,lk) :        capital holding decision kp
!
! Output:
!		solve the invariant distribution of A
!       solve the complete market
!       simulate the model
!       generate the results of simulation
!       Run FH regression
!
!       fhcoef       :        regression coefficient of FH puzzle for each simulation
!       statistics for fhcoef
!
!   N:                        number of simulations
!   T:                        number of years to be simulated
!   NC:                       number of countries to be simulated
!
! Author: Bai and Zhang
! February 21, 2009
!*******************************************************

USE SimuVar

implicit none

! Simulation results
integer, dimension(T, nc)   :: isimua
real*8, dimension(T, nc)    :: simua
real*8, dimension(1, nc)    :: simuk0, simub0
real*8, dimension(T1, nc)   :: simuk, simuc, simuy, simui, simus, simub, simukk

! Statistics of simulation
real*8, dimension(nc)       :: fhsy, fhiy
real*8,dimension(N)         :: setb, stdb                     
real*8                      :: b0, b1, tb0, tb1
real*8, dimension(12)       :: stat1, tstat1
real*8, dimension(N,NC)     :: settb, stdtb, settc

! Working arrays
integer                    :: i, j, m, ii, jj, mm, ia, iap, s, it, cut
real*8                     :: gridb, temp, temp1, tmp, tmp1, mtb, mtc
real*8                     :: esig, dnordf, dist1,kyratio(N,nc),mky(N),stdky(N)
real*8, dimension(la,la)   :: exd
character(24)              :: dd1
real*8     :: afhiy(N),afhsy(N),corr(N),sfhiy(N),sfhsy(N)

! welfare cost
real*8:: twelc(T1,nc),ttwelc(nc),cc(nc),welc(N),tb(N),ta(N),mmc(N)
real*8:: sgdp(NC),csys(N),mcsys,stdcsys,csyi(N),mcsyi, stdcsyi
real*8:: dsav(N),dinv(N),dgdp(N) ! dispersions
real*8:: per(3,2),tgdp(NC),tfhiy(NC),tfhsy(NC),mgdp(N)
real*8:: mcay(nc),mtay(nc),cay(N),tay(N),scay(N),stay(N)
integer:: l5,u5

external completeresult

! Input files
open(unit=1,  file=trim(dirin)//trim(sshock)) 
 
!****************************************************
if (reada==1) then
	! Read in state spaces
	do i=1,la
		read(1,*) a(i)
	end do

	do i=1,la
		do j=1,la
			read(1,*) p(i,j)
		end do
	end do

else
	! create a transition matrix for an AR(1) process
	! e(t) = mu(1-rho)+ rho* e(t-1) + sigmau*u(t)   where u ~ N(0,1)

	esig = (sigmau**2.0/(1.0 - rho**2.0))**0.5

    ! Have the range of e within m standard deviations
	do i = 1, la
		a(i)  = amiu - istd * esig + (i-1)*((2.0*istd*esig)/(la-1))
	end do

	p = 0.0
	do j = 1,la     ! j is the current state
		do i = 1,la ! i is the future state
			if (i == 1) then
				dist1  = a(i+1) - a(i)
				p(j,i) = dnordf((a(i) - amiu - rho * (a(j)-amiu)+ dist1/2.0)/sigmau)
			elseif (i == la) then
				dist1  = a(i) - a(i-1)
				p(j,i) = 1.0 - dnordf((a(i) - amiu - rho*(a(j)-amiu)-dist1/2.0)/sigmau)
			else
				dist1  = a(i) - a(i-1)
				p(j,i) = dnordf((a(i) - amiu - rho*(a(j)-amiu) + dist1/2)/sigmau) - dnordf((a(i) - amiu - rho*(a(j)-amiu) - dist1/2)/sigmau)
			end if
		end do
	end do
	a   = exp(a)/3.0
	
	print *, 'rho    = ', rho
	print *, 'sigmau = ', sigmau
	
	! Clean the transition matrix
	do i=1,la
		p(i,i) = p(i,i) + (1.0-sum(p(i,:)))
	!	print *, sum(p(i,:))
	end do	
end if

do i=1, la
	Ea(i)=sum(p(i,:)*a)
end do


!*******************************************************************
! Solve the complete market decision rule and invariant distribution
call completeresult

!******************************************************
! Start simulating the model
! Cumulative sum of transition matrix

cumpai(:,1)=p(:,1)
do i=2,la
	cumpai(:,i)=cumpai(:,i-1)+p(:,i)
end do

!******************************************************
do s=1,N
	call Simu(simua, isimua, simuk0, simub0)

	do i=1,nc
		simukk(1,i)    = simuk0(1,i)
		simub(1,i)     = simub0(1,i)
		temp           = a(isimua(1,i)) * simukk(1,i)**alpha + (1.0-delta)*simukk(1,i)
		simuc(:,i)     = (1.0 - beta1) * (temp + beta1 * inc(isimua(1,i)) + simub(1,i))
								! Consumption is constant over life depends on lifetime income
		if (T>T1) then ! If we need to cut the earlier periods
			
			do j = 1,T-T1
				simukk(1,i) = k(isimua(j,i))
				temp        = simua(j+1,i) * simukk(1,i)**alpha + (1.0-delta) * simukk(1,i)
				simub(1,i)  = simuc(1,i)/(1.0-beta1) - temp - beta1 * inc(isimua(j+1,i))  ! Next period bond holding given a' realizes
			end do
			
			! Recompute the consumption
			temp        = a(isimua(T-T1+1,i))* simukk(1,i)**alpha + (1.0-delta)*simukk(1,i)
			simuc(:,i)  = (1.0 - beta1) * (temp + beta1 * inc(isimua(T-T1+1,i)) + simub(1,i))
			
			do j = T-T1+1,T
				it                 = j - (T-T1)
				simuy(it,i)        = a(isimua(j,i)) * simukk(it,i)**alpha
				simuk(it,i)        = k(isimua(j,i))
				if (j<T) then
					simukk(it+1,i) = simuk(it,i)
					temp           = simua(j+1,i) * simukk(it+1,i)**alpha + (1.0-delta)*simukk(it+1,i)
					simub(it+1,i)  = simuc(it,i)/(1.0-beta1) - temp - beta1 * inc(isimua(j+1,i)) ! Next period bond holding given a' realizes
				end if
				simui(it,i)        = simuk(it,i) - (1.0-delta)*simukk(it,i)
				
				if (isav == 1) then
					!Savings is defined as GDP minus consumption
					simus(it,i)        = simuy(it,i) - simuc(it,i)
				else
					!Savings is defined as GNP minus consumption
					simus(it,i)        = simuy(it,i) - simuc(it,i) + (1.0/beta1 - 1.0)*simub(it,i)
				end if

			end do                         ! end of simulation for T
		else
			
			temp            = a(isimua(1,i))* simuk(1,i)**alpha + (1.0-delta)*simuk(1,i)
			simuc(1:T,i)    = (1.0 - beta1) * (temp + beta1 * inc(isimua(1,i)) + simub(1,i))
			simukk(1,i)     = simuk0(1,i)
			simub(1,i)      = simub0(1,i)
			do j=1,T
				simuy(j,i)  = a(isimua(j,i)) * simukk(j,i)**alpha
				simuk(j,i)  = k(isimua(j,i))
				if (j<T) then
					simukk(j+1,i) = simuk(j,i)
					temp          = simua(j+1,i) * simukk(j+1,i)**alpha + (1.0 - delta)*simukk(j+1,i)
					simub(j+1,i)  = simuc(j,i)/(1.0-beta1) - temp - beta1 * inc(isimua(j+1,i)) ! Next period bond holding given a' realizes
				end if
				simui(j,i)  = simuk(j,i) - (1.0-delta)*simukk(j,i)
				simus(j,i)  = simuy(j,i) - simuc(j,i) 
			end do
		end if
		kyratio(s,i) = sum(simuk(:,i)/simuy(:,i))/real(T)	
			
		mcay(i)=sum(abs(simub(2:T1,i)-simub(1:T1-1,i))/simuy(1:T1-1,i))/real(T1-1)
		mtay(i)=sum(abs(simub(:,i))/simuy(:,i))/real(T1)
	end do                             ! end of simulation for nc 

	! welfare cost	
	twelc=(simuc**(1.0-sigma)-1)/(1.0-sigma)
	ttwelc=0.0
	do i=1,T1
		ttwelc=ttwelc+(beta1**(i-1))*twelc(i,:)
	end do
	cc=((ttwelc*(1-beta1)/(1-beta1**T1))*(1-sigma)+1)**(1.0/(1-sigma))
	welc(s)=sum(cc)/real(NC)
	mmc(s)=sum(simuc)/real(NC*T1)
	
	! capital flow
	call meanstd(NC,mcay,cay(s),scay(s))
	call meanstd(NC,mtay,tay(s),stay(s))
	
	! Cross section regression for each year
	mky(s)    = sum(kyratio(s,:))/real(nc)
	stdky(s)  = sqrt(sum((kyratio(s,:)-mky(s))**2.0)/real(nc))
	

	fhiy  = sum(simui/simuy,1)/real(T1)
	fhsy  = sum(simus/simuy,1)/real(T1)
	afhiy(s)=sum(fhiy)/real(nc)
	afhsy(s)=sum(fhsy)/real(nc)
	sfhiy(s)=sqrt(sum((fhiy-afhiy(s))**2)/real(nc-1))
	sfhsy(s)=sqrt(sum((fhsy-afhsy(s))**2)/real(nc-1))

    	call drline(nc,fhsy, fhiy, b0, b1, stat1)
	setb(s)  = b1
	stdb(s)  = stat1(7)
	corr(s)  = stat1(5)
	
		!********************
	! correlation between output and investment and savings
	! and  dispersion

	sgdp=100*((simuy(T1,:)/simuy(1,:))**(1/real(T1-1))-1)
	
	call drline(nc,sgdp,fhiy, b0, b1, stat1)
	csyi(s) = stat1(5)
	dinv(s) = sqrt(stat1(4))
	dgdp(s) = sqrt(stat1(3))
	mgdp(s) = stat1(1)
	
	call drline(nc,sgdp,fhsy, b0, b1, stat1)
	csys(s) = stat1(5)
	dsav(s) = sqrt(stat1(4))
	call  dsvrgn(NC,fhiy,tfhiy)
	call  dsvrgn(NC,fhsy,tfhsy)
	call  dsvrgn(NC,sgdp,tgdp)
	
	l5=floor(NC*0.10)
	u5=floor(NC*0.90)
	
	per(1,1)=sum(tfhiy(1:l5))/real(l5)
	
	per(1,2)=sum(tfhiy(u5:NC))/real(NC-u5+1)
	per(2,1)=sum(tfhsy(1:l5))/real(l5)
	per(2,2)=sum(tfhsy(u5:NC))/real(NC-u5+1)
	per(3,1)=sum(tgdp(1:l5))/real(l5)
	per(3,2)=sum(tgdp(u5:NC))/real(NC-u5+1)

end do                                 ! end of N

!**********************************************
call meanstd(N,csys,mcsys,stdcsys)
call meanstd(N,csyi,mcsyi,stdcsyi)

write (*,12)
write (*, *)  '******** Complete Markets Model Simulation Report ********'
write (*, 12)
write (*, 24) '  # of simulation:    ', N
write (*, 20) '  # of periods:       ', T1
write (*, 20) '  # of countries:     ', NC
write (*,12)

write(*, 16)  '  FH coef:            ', sum(setb)/real(N),sum(stdb)/real(N)
write(*, 16)  '  Capital flow        ', sum(cay)/real(N),sum(scay)/real(N)
write(*, 18)  '  TA/Y                ', sum(tay)/real(N),sum(stay)/real(N)
write(*, *) '-------------------------------' 
write(*,*)  ' Cross-country mean:           '
write(*,14)  '   Investment rate:            ', sum(afhiy)/real(N)
write(*,14)  '   Saving rate:                ', sum(afhsy)/real(N)
write(*,14)  '   GDP growth rate:            ', sum(mgdp)/real(N)
write(*,*)
write(*, *) ' Cross-country correlation:    '
write(*, 16) '   GDP growth and average investment rate: ', mcsyi,stdcsyi
write(*, 16) '   GDP growth and average savings rate:    ', mcsys,stdcsys
write(*,*)     
write(*, *) ' Cross-country dispersion:   '
write(*, 14) '   Std of average investment rate: ', sum(dinv)/real(N)
write(*, 14) '   Std of average savings rate:    ', sum(dsav)/real(N)
write(*, 14) '   Std of GDP growth rate:        ', sum(dgdp)/real(N)
write(*,*)     
write(*, *) ' Cross-country dispersion:   '
write(*,*)  '        Investment      Saving       GDP growth'
write(*,902)  'Min 10  ', per(1,1),per(2,1),per(3,1)
write(*,902)  'Max 10  ', per(1,2),per(2,2),per(3,2)
write (*, 12)
write(*, *) '***********************************************'


12 format(/)
14 format(a40,F6.3)
16 format(a40,F7.3,' (',F4.3,')')
18 format(a40,F7.3,' (',F5.2,')')
20 format(a40,I2)
24 format(a40,I5)
22 format(a40,F8.3)
902 format(A10,f7.3,'       ', f7.3,'       ',f7.3)




END program SimuComplete






































