subroutine Simu(simua, isimua, simuk0, simub0)

!*******************************************************
! Simulation for the complete market:
! Input:
!       Productivity shock process A and P
!       Invariant distribution over states (A,K,B): miu (in CRS format)
!
!       miu(la*lk*lb):        stationary distribution
!
! Output:
!       Following simulation results for NC countries and T period
!       (1) Initial year state (A0, K0, B0)
!       (2) A(t) for t = 2: T-1
!
!       simua(NC,T):                simulated productivity process
!       simuk0(NC,1):               simulated capital stock
!       simub0(NC,1):               simulated bonds holding
!
! Parameters
!       T:                          number of years to be simulated
!       NC:                         number of countries to be simulated
!       la:                         number of productivity levels
!       lk:                         number of capital grid
!       lb:                         number of bond grid
!
!*******************************************************

USE SimuVar

implicit none

! Results
integer, intent(out), dimension(T,NC)   :: isimua
real, intent(out), dimension(T,NC)      :: simua
real, intent(out), dimension(1,NC)      :: simuk0,simub0

! Working arrays
integer                                 :: i, j, ia, ib, ik, s1, res, m, tt

integer, dimension(NC)                  :: state
real, dimension(NC)                     :: tran1
real, dimension(T)                      :: tran
real, dimension(T,NC)                   :: ranp
real, dimension(T1,NC)                  :: tsimua

real, dimension(la)                     :: invp, cuminvp
complex, dimension(la)                  :: eval
complex, dimension(la,la)               :: evec
real, dimension(la)                     :: reval
real, dimension(la,la)                  :: revec, tmpm, invevec, tmpm1, tmpmm

!*******************************************************
! Simulation
! Step (1)
! Generate T*nc random numbers
 
if (N==1) then               ! If only simulate once, use fixed seed to get constant result for testing
	!s1  =  100535           ! Select 5
	!call rnset(s1)
	call rnget(s1)
	!print *, 'rseed: ',s1
end if

! Solve invariant distribution of a according to transition matrix P
! First find the eigenvalue and eigenvectors
call devcrg(la,p,la,eval,evec,la)


reval  = real(eval)
revec  = real(evec)
!print *, 'maximum eigenvalue=', reval(1)
!if (reval(1)/=1.d0  ) print *, 'The transition matrix does not have an invariant distribution!'
tmpm      = 0.0
tmpm(1,1) = 1.0

call dlinrg(la,revec,la,invevec,la)

tmpm1     = matmul(tmpm,invevec)
tmpmm     = matmul(revec,tmpm1) 

invp      = tmpmm(1,:)   ! Invariant distribution

cuminvp(1)     = invp(1)
do i=2,la
	cuminvp(i) = cuminvp(i-1) + invp(i)
end do


! Draw random number for initial state  according to invariant distribution invp
call drnun(NC,tran1)
do i=1, NC
	do m = 1,la
		if (cuminvp(m) >= tran1(i)) then
			isimua(1,i) = m
			simua(1,i)  = a(m)
			exit
		end if
	end do	
	simuk0(1,i)         = k(isimua(1,i))  ! Initial capital 

end do

simub0(1,NC/2+1:NC)     = -0.0

! Random number for the following state a'
tran = 0.0
do i=1,nc
	call drnun(T-1,tran)
	ranp(:,i)=tran
end do

! Find following states for each country
do i = 1,nc              
	do j = 2,T                       ! Iterate over periods 2 : T
		do m = 1,la			
			if (cumpai(isimua(j-1,i),m) >= ranp(j-1,i)) then
				isimua(j,i) = m
				simua(j,i)  = a(m)
				exit
			end if
		end do
	end do                         ! end of simulation for T
end do                             ! end of simulation for nc 



END subroutine Simu









