Program CostComplete

!*******************************************************
! Simulate and test for complete market results with CAPITAL ADJUSTMENT COST
!
! Algorithm:
!       First solve the invariant distribution of a (prod shocks)
!          We assume that each country start to the k0 according to a0 (i.e. a(-1) = a0)
!       It is hard to calibrate exactly b0. So start with b0=0 for all series. Then simulate
!          for long periods then drop the first periods and leave only 44 periods as in the data.
!
! Input:
!       Productivity shock process A and P
!       Decision rule: gb, gk
!       State grids  : k,  b
!
!       gb(la*lb,lk) :        bond holding decision bp
!       gk(la*lb,lk) :        capital holding decision kp
!
! Output:
!		solve the invariant distribution of A
!       solve the complete market
!       simulate the model
!       generate the results of simulation
!       Run FH regression
!
!       fhcoef       :        regression coefficient of FH puzzle for each simulation
!       statistics for fhcoef
!
!   N:                       number of simulations
!   T:                        number of years to be simulated
!   NC:                     number of countries to be simulated
!   T1:                     number of years left after drop the first T-T1 periods
!
!   Author: Bai and Zhang
!   March 20, 2009
!*******************************************************


USE CostVar

implicit none

! Simulation results
integer, dimension(T, nc) :: isimua
integer, dimension(1, nc) :: isimuk0
real, dimension(T, nc)    :: simua
real, dimension(1, nc)    :: simuk0, simub0
real, dimension(T1, nc)   :: simuk, simuc, simuy, simui, simus, simub, simukk

real, dimension(2)        :: valk
integer, dimension(2)     :: indk


! Statistics of simulation
real, dimension(nc)       :: fhsy, fhiy
real, dimension(N)        :: setb, stdb                     
real                      :: b0, b1
real, dimension(12)       :: stat1

real, dimension(nc)       :: stdinv, stdgdp
real, dimension(N)        :: relstd                     
real                      :: mrelstd
real                      :: minv(T1-1,nc),mgdp(T1-1,nc),viy(nc)


! Working arrays
integer                   :: i, j, m, ii, jj, mm, s, it, cut
real                      :: gridk, gridb, temp, temp1, tmp, tmp1, kp,tcost
real, dimension(la,la)    :: exd


!time-series correlation
real                         :: tsimus(T1,NC),tsimui(T1,NC),tsc(T1),hsimus(T1),hsimui(T1),msfh(NC)
real                         :: sfh(N),ssfh(N),tsimuy(T1,NC),hsimuy(T1),sviy(NC)
integer        :: ins,inn,u2
real,parameter :: hw=100


!external compgk, compw, dist, Simuct
external solvegk

! Input files
open(unit=1,  file=trim(dirin)//'shock9.dat') 
 
! Output of complete market decision rules
open(unit=3,  file=trim(dirout)//'gc.out')    
open(unit=5,  file=trim(dirout)//'gk.out')  
open(unit=7,  file=trim(dirout)//'gw.out') 
open(unit=9,  file=trim(dirout)//'kgrid.dat')    

!****************************************************
! Read in state spaces
do i=1,la
	read(1,*) a(i)
end do

do i=1,la
	do j=1,la
		read(1,*) p(i,j)
	end do
end do

! Clean the transition matrix
do i=1,la
	p(i,i) = p(i,i) + (1.0-sum(p(i,:)))

end do

do i=1, la
	Ea(i)=sum(p(i,:)*a)
end do

!*******************************************************************
! Initialize the capital space

gridk=(kmax-kmin)/(lk-1)
do i=1,lk
	K(i)=kmin+(i-1)*gridk
end do
rewind(9)
do i = 1,lk
	write(9,*) K(i)
end do
rewind(9)

!*******************************************************************
R  = 1.0/beta1

! Solve capital decision rule gk(a,k)
print *, 'First compute the capital decision rule'
call compgk

! Solve for the income W(a,k)
print *, 'Second compute the income'
call compw

! Compute the invariant distribution over (a,k)
print *, 'Third compute the invariant distribution'
call dist


!******************************************************
! Start simulating the model
! Cumulative sum of transition matrix

print *, 'Start simulating the model'
cumpai(:,1)      = p(:,1)
do i=2,la
	cumpai(:,i)  = cumpai(:,i-1)+p(:,i)
end do

cummiu(1)  = miu(1)
do i=2,la*lk
	cummiu(i)    = cummiu(i-1) + miu(i)
end do

!******************************************************
print *, 'Start running FH cross section regressions:'
ins=0
do s=1,N
	call Simuct(simua, isimua, simuk0, isimuk0, simub0)

	do i=1,nc
		simukk(1,i)    = simuk0(1,i)
		simub(1,i)     = simub0(1,i)
		simuc(:,i)     = (1.0 - beta1)*(gw(isimua(1,i),isimuk0(1,i)) + simub(1,i))
		! Consumption is constant over life depends on lifetime income

		if (T>T1) then ! If we need to cut the earlier periods
			
			do j = 1,T-T1
				call distbase(simukk(1,i), valk, indk)

				kp              = 0.0
				do jj = 1, 2
					kp          = kp + gk(isimua(j,i), indk(jj)) * valk(jj) 
				end do
				simukk(1,i)     = kp
			end do
					
			do j = T-T1+1,T
				it              = j - (T-T1)

				simuy(it,i)     = a(isimua(j,i)) * simukk(it,i)**alpha  
				
				call distbase(simukk(1,i), valk, indk)

				kp              = 0.0
				do jj = 1, 2
					kp          = kp + gk(isimua(j,i), indk(jj)) * valk(jj) 
				end do
				simuk(it,i)     = kp

				if (j<T) then
					simukk(it+1,i) = simuk(it,i)
				end if
				tcost=gama*simukk(it,i)*((simuk(it,i)-simukk(it,i))/simukk(it,i))**2.0/2.0
				simui(it,i) = simuk(it,i) - (1.0 - delta) * simukk(it,i)+tcost

				if (isav == 1) then
					!Savings is defined as GDP minus consumption
					simus(it,i)        = simuy(it,i) - simuc(it,i)
				else
					!Savings is defined as GNP minus consumption
					simus(it,i)   = simuy(it,i) - simuc(it,i) + (1.0/beta1 - 1.0)*simub(it,i)
					
				end if
				tsimui(it,i) = simui(it,i)*gt**(real(it-1))
				tsimus(it,i) = simus(it,i)*gt**(real(it-1))
				tsimuy(it,i)= simuy(it,i)*gt**(real(it-1))

			end do     ! end of simulation for T
			
		else
			temp            = a(isimua(1,i))* simuk(1,i)**alpha + (1.0-delta)*simuk(1,i)
			simuc(1:T,i)    = (1.0 - beta1) * (temp + beta1 * inc(isimua(1,i)) + simub(1,i))
			simukk(1,i)     = simuk0(1,i)
			simub(1,i)      = simub0(1,i)
			do j=1,T
				simuy(j,i)  = a(isimua(j,i)) * simukk(j,i)**alpha 
				simuk(j,i)  = k(isimua(j,i))
				if (j<T) then
					simukk(j+1,i) = simuk(j,i)
					temp          = simua(j+1,i) * simukk(j+1,i)**alpha + (1.0 - delta)*simukk(j+1,i)
					simub(j+1,i)  = simuc(j,i)/(1.0-beta1) - temp - beta1 * inc(isimua(j+1,i)) ! Next period bond holding given a' realizes
				end if
				simui(j,i)  = simuk(j,i) - (1.0-delta)*simukk(j,i)
				simus(j,i)  = simuy(j,i) - simuc(j,i) 
				
				
			end do
		end if
		!time-series saving-investment correlation
		call hpfilter(tsimus(:,i),tsc,hsimus,T1,hw)
		call hpfilter(tsimui(:,i),tsc,hsimui,T1,hw)
		call hpfilter(tsimuy(:,i),tsc,hsimuy,T1,hw)
		call drline(T1,hsimus(1:T1),hsimui(1:T1), b0, b1, stat1)
		msfh(i) = stat1(5)	
	end do                             ! end of simulation for nc 
	
	relstd(s)=sum(viy)/real(nc)
	call meanstd(nc,msfh,sfh(s),ssfh(s))
	
	! Cross section regression for each year
	fhiy  = sum(simui/simuy,1)/real(T1)
	fhsy  = sum(simus/simuy,1)/real(T1)

	call drline(nc,fhsy, fhiy, b0, b1, stat1)
	setb(s)  = b1
	stdb(s)  = stat1(7)
	
	minv  = simui(2:T1,:)/simui(1:T1-1,:)
	mgdp  = simuy(2:T1,:)/simuy(1:T1-1,:)

	do i=1,nc
		call drline(T1-1,minv(:,i), mgdp(:,i), b0, b1, stat1)
		viy(i) = sqrt(stat1(3))/sqrt(stat1(4))
	end do
	call  dsvrgn(NC,viy,sviy)
	
	u2=floor(NC*0.95)
	relstd(s)=sum(sviy(1:u2-1))/(u2-1)
	
end do                                 ! end of N

mrelstd   = sum(relstd)/real(N)
print *, 'Volatility of investment relative to volatility of output:'
print *,  mrelstd
print *, 'Time-series saving-investment correlation'
print *, sum(sfh)/real(N),'(',sum(ssfh)/real(N),')'

!**********************************************
3  format(f8.4)

print *, 'Mean of regression coefficients:'
print 3, sum(setb)/real(N)


END program CostComplete






