subroutine Simuct(simua, isimua, simuk0, isimuk0, simub0)

!*******************************************************
! Simulation for the complete market:
! Input:
!       Productivity shock process A and P
!       Invariant distribution over states (A,K,B): miu (in CRS format)
!
!       miu(la*lk*lb):        stationary distribution
!
! Output:
!       Following simulation results for NC countries and T period
!       (1) Initial year state (A0, K0, B0)
!       (2) A(t) for t = 2: T-1
!
!       simua(NC,T):                simulated productivity process
!       simuk0(NC,1):               simulated capital stock
!       simub0(NC,1):               simulated bonds holding
!
! Parameters
!       T:                          number of years to be simulated
!       NC:                         number of countries to be simulated
!       la:                         number of productivity levels
!       lk:                         number of capital grid
!       lb:                         number of bond grid
!
!*******************************************************



USE CostVar

implicit none

! Results
integer, intent(out), dimension(T,NC)   :: isimua
integer, intent(out), dimension(1,NC)   :: isimuk0
real, intent(out), dimension(T,NC)      :: simua
real, intent(out), dimension(1,NC)      :: simuk0,simub0

! Working arrays
integer                    :: i, j, s1, res, m, tt

integer, dimension(NC)     :: state
real, dimension(NC)        :: tran1
real, dimension(T)         :: tran
real, dimension(T,NC)      :: ranp
real, dimension(T1,NC)     :: tsimua

real, dimension(la)        :: invp, cuminvp
complex, dimension(la)     :: eval
complex, dimension(la,la)  :: evec
real, dimension(la)        :: reval
real, dimension(la,la)     :: revec, tmpm, invevec, tmpm1, tmpmm

!*******************************************************
! Simulation

if (itest == 0) then
	! Draw random number for initial state  according to invariant distribution miu
	! Step (1) Generate 1*nc random numbers

	call drnun(NC,tran1)
	do i=1, NC
		do m = 1,la*lk
			if (cummiu(m)   >= tran1(i)) then
				ia           = ceiling(real(m)/real(lk))
				ik           = m - (ia-1)*lk
				isimua(1,i)  = ia
				simua(1,i)   = a(ia)
				simuk0(1,i)  = k(ik)           ! Initial capital 
				isimuk0(1,i) = ik
				exit
			end if
		end do	
		simub0(1,i)          = 0.0             ! Initital bonds is zero!	
	end do

	!isimua(1,:)             = la/2
	!simua(1,:)              = a(la/2)
	!simuk0(1,:)             = k(la/2)
	!simub0(1,:)             = 0.0

	! Random number for the following state a'
	tran   = 0.0
	do i   = 1, nc
		call drnun(T-1,tran)
		ranp(:,i)   =  tran
	end do

	! Find following states for each country
	do i   = 1,nc              
		do j = 2,T                       ! Iterate over periods 2 : T
			do m = 1,la			
				if (cumpai(isimua(j-1,i),m) >= ranp(j-1,i)) then
					isimua(j,i)  = m
					simua(j,i)   = a(m)
					exit
				end if
			end do
		end do                           ! End of simulation for T
	end do                               ! End of simulation for nc 

elseif (itest == 1) then

	! Experiment by letting all processes start with the same initial conditions
	isimua(1,:)  = la/2-2
	simua(1,:)   = a(la/2-2)
	
	simuk0(1,:)         = k(la/2)  ! Initial capital 
	simub0(1,:)         = 0.0             ! Initital bonds is zero!	

	! Random number for the following states a'
	tran = 0.0
	do i=1,nc
		call drnun(T-1,tran)
		ranp(:,i)=tran
	end do

	! Find following states for each country
	do i=1,nc              
		do j=2,T                       ! Iterate over periods 2 : T
			do m = 1,la			
				if (cumpai(isimua(j-1,i),m)>=ranp(j-1,i)) then
					isimua(j,i) = m
					simua(j,i)  = a(m)
					exit
				end if
			end do
		end do                         ! end of simulation for T
	end do                             ! end of simulation for nc 

else

	! Experiment by simulate the shock close to the data process
	isimua(1,:)         = la/2
	simua(1,:)          = a(la/2)
	
	simuk0(1,:)         = k(la/2)  ! Initial capital 
	simub0(1,:)         = 0.0             ! Initital bonds is zero!	
	isimuk0(1,:)        = la/2

	! Random number for the following states a'
	if (T>T1) then
		tran = 0.0
		do i=1,nc
			call drnun(T-1,tran)
			ranp(:,i)=tran
		end do

		! Find following states for each country
		do i=1,nc              
			do j=2,T-T1                       ! Iterate over periods 2 : T
				do m = 1,la			
					if (cumpai(isimua(j-1,i),m)>=ranp(j-1,i)) then
						isimua(j,i) = m
						simua(j,i)  = a(m)
						exit
					end if
				end do
			end do                         ! end of simulation for T
		end do 
	end if
	
	rewind(33)
	do i=1,NC
		do j=T-T1+1,T
			read(33,*) tsimua(j-(T-T1),i)
			tsimua(j-(T-T1),i) = exp(tsimua(j-(T-T1),i))/50.0
			tt  = minloc(abs(a-tsimua(j-(T-T1),i)),1)
			simua(j,i)  = a(tt)
			isimua(j,i) = tt
		end do
		!simuk0(1,i)         = k(isimua(1,i))  ! Initial capital 
		if (T>T1) then
			simua(T-T1,i)   = simua(T-T1+1,i)
			isimua(T-T1,i)  = isimua(T-T1+1,i)
		end if
	end do
	rewind(33)

	!simub0(1,:)         = 0.0             ! Initital bonds is zero!	

	
end if

END subroutine Simuct
