Program SimuEnforce

!**************************************************************************
! Simulate and test for complete market with enforcement constaint results
!
! Input:
!       Productivity shock process A and P5
!       Invariant distribution over states (A,W): miu (in CRS format)
!       Decision rule: gw, gk
!       State grids  : W
!
!       miu(la*lw):        stationary distribution
!       gb(la,lw) :        bond holding decision bp
!       gk(la,lw) :        capital holding decision kp
!
! Output:
!       simulate the model
!       generate the results of simulation
!       Run FH regression
!
!       fhcoef       :        regression coefficient of FH puzzle for each simulation
!       statistics for fhcoef
!
!   NSS:                      number of simulations
!   T:                        number of years to be simulated
!   NC:                       number of countries to be simulated
!
! Author: Bai and Zhang
! February 21, 2009
!*******************************************************


USE simuvar

implicit none

! Simulation results
integer, dimension(T+1, nc)  :: isimua
real, dimension(T+1, nc)     :: simua
real, dimension(1, nc)       :: simuk0, simuw0
real, dimension(T, nc)       :: simukk, simuww, simuk, simuw, simuh, simuc, simuy, simui, simus, simubb

! Statistics of simulation
real, dimension(nc)          :: fhsy, fhiy
real,dimension(NSS)          :: setb, stdb                     
real                         :: b0, b1
real, dimension(12)          :: stat1,mystat
character(24)                :: dd1

! Working arrays
integer                      :: i, j, m, ii, jj, mm, itmp, s, cut, iwt
real                         :: temp, temp1, h, kp, wp, tmp, tmp1,b

real, dimension(2)           :: valk,valw
integer, dimension(2)        :: indk,indw
real                         :: cay(NSS),tay(NSS),scay(NSS),stay(NSS)
! risksharing
real                         :: grc(T-1,NC), gry(T-1,NC), grmc(T-1),grmy(T-1),mc(T),my(T)
real                         :: pdifb(2),mpdfib,stdpdifb,spdifb(NSS)
real, dimension((T-1)*nc)              :: pgrc,pgry,pgrmc,pgrmy
!time-series correlation
real                         :: tsimus(T,NC),tsimui(T,NC),tsc(T),hsimus(T),hsimui(T),sfh(NSS)
real                         :: msfh(NC),ssfh(NSS),hw
integer                    :: tlw

!dispersion
real*8:: sgdp(NC),csys(NSS),mcsys,stdcsys,csyi(NSS),mcsyi, stdcsyi
real*8:: dsav(NSS),dinv(NSS),dgdp(NSS) ! dispersions
real*8:: per(3,2),tgdp(NC),tfhiy(NC),tfhsy(NC),mgdp(NSS),afhiy(NSS),afhsy(NSS)
real*8:: mcay(NC), mtay(NC)
integer:: l5,u5


external Simu, simubase

! Input files
open(unit=4,  file=trim(sfolder)//'shock.dat')    
open(unit=6,  file =trim(sfolder)//'kgrid.dat')
open(unit=8,  file =trim(sfolder)//'wgrid.dat')

open(unit=10, file=trim(sfolder)//'gk.dat')    
open(unit=12, file=trim(sfolder)//'gw.dat')    
open(unit=14, file=trim(sfolder)//'gh.dat')    
open(unit=16, file=trim(sfolder)//'den.dat')
open(unit=18, file=trim(sfolder)//'sparsep.dat')
open(unit=20, file=trim(sfolder)//'gbb.dat')
open(unit=22, file=trim(sfolder)//'param1.dat')
open(unit=24, file=trim(sfolder)//'param.dat')

!****************************************************
! Read in shocks
hw=100
rewind(4)
do i = 1,la
	read(4,*) A(i)
end do
do i=1,la
	do j=1,la
		read(4,*) p(i,j)
	end do
end do
do i=1,la
	Ea(i)=sum(p(i,:)*A)
end do
rewind(4)
rewind(22)
read(22,*)
read(22,*) beta

rewind(24)
read(24,*)
read(24,*) tlw

if (tlw /= lw) then
	write(*,*) 'Please change lw in the simuvar.f90 to: ', tlw, ' and rerun the program!'
	stop
end if

! Read shock process in CRS format
rewind(18)
read (18,*) indexp
allocate(pp(indexp), jpp(indexp))
do i = 1, indexp
	read (18,*) pp(i)
end do
do i = 1, indexp
	read (18,*) jpp(i)
end do
do i = 1, la+1
	read (18,*) ipp(i)
end do
rewind(18)

! Read in the K and W space
rewind(6)
rewind(8)
do i=1,lk
	read(6,*) K(i)
end do
do i=1,lw
	read(8,*) W(i)
end do
rewind(6)
rewind(8)

!******************************************************
! Read in decision rules and invariant distribution
rewind(10)
rewind(12)
rewind(14)
rewind(16)
rewind(20)
do i =1,la
	do j =1,lw
		read(10, *) gk(i,j)
		read(14, *) gh(i,j)
		do m =1,lk
			read(16,*) den((i-1)*lw*lk+(j-1)*lk+m)	
			read(20,*) gb(i, j, m)	
			if (den((i-1)*lw*lk+(j-1)*lk+m)/=0) itmp=(i-1)*lw*lk+(j-1)*lk+m
		end do
	end do
end do
do i = 1, la
	do ii = ipp(i), ipp(i+1)-1
		do j = 1,lw
			read(12, *) gw(i,j,jpp(ii))
		end do
	end do
end do

den(itmp)   = den(itmp) + (1.0 - sum(den))

!*******************************************************************
! Start simulating the model
! Cumulative sum of transition matrix
cumpai(:,1)     = p(:,1)
do i=2,la
	cumpai(:,i) = cumpai(:,i-1)+p(:,i)
end do

cummiu(1)       = den(1)
do i=2,la*lk*lw
	cummiu(i)   = cummiu(i-1)+den(i)
end do

!******************************************************
do s=1,NSS
	call Simu(simua, isimua, simuk0, simuw0)
	do i=1,nc
		do j=1,T
			if (j==1) then 
				simukk(j,i)     = simuk0(1,i)
				simuww(j,i)     = simuw0(1,i)
			else
				simukk(j,i)     = simuk(j-1,i)
				simuww(j,i)     = simuw(j-1,i)
			end if
			
			call simubase(simukk(j,i),simuww(j,i), valk, valw, indk, indw)

			! First translate w  into b
			b                   = 0.0
			do ii = 1,2     ! Index of W
				do jj = 1,2 ! Index of K
					b           = b  + gb(isimua(j,i),indw(ii),indk(jj))*valk(jj)*valw(ii)
				end do
			end do
			simubb(j,i)         = b

			! Second compute the policy decision
			kp                  = 0.0
			wp                  = 0.0
			h                   = 0.0
	
			do ii = 1, 2
				kp          = kp + gk(isimua(j,i),indw(ii)) * valw(ii)
				wp          = wp + gw(isimua(j,i),indw(ii),isimua(j+1,i)) * valw(ii) 
				h           = h  + gh(isimua(j,i),indw(ii)) * valw(ii) 
			end do
			
			simuk(j,i)          = kp
			simuw(j,i)          = wp
			simuh(j,i)          = h

			simuy(j,i)          = simua(j,i) * simukk(j,i)**alpha
			simui(j,i)          = simuk(j,i) - (1.0 - delta) * simukk(j,i)
			simuc(j,i)          = cc(h)
			simus(j,i)          = simuy(j,i) - simuc(j,i)
			
			tsimus(j,i)         =simus(j,i)*gt**(real(j-1))
			tsimui(j,i)         =simui(j,i)*gt**(real(j-1))

			
		end do                         ! end of simulation for T
		! time-series saving-investment correlation
		call hpfilter(tsimus(:,i),tsc,hsimus,T,hw)
		call hpfilter(tsimui(:,i),tsc,hsimui,T,hw)
		
		call drline(T,hsimus(1:T),hsimui(1:T), b0, b1, stat1)
		msfh(i) = stat1(5)
		! capital flows
		mcay(i) =sum((abs(simubb(2:T,i)-simubb(1:T-1,i))/simuy(1:T-1,i)))/real(T-1)
		mtay(i)=sum(abs(simubb(:,i))/simuy(:,i))/real(T)

		
	end do                             ! end of simulation for nc 
	
	! capita flows and time-series fh coef
	call meanstd(NC, mcay, cay(s),scay(s))
	call meanstd(NC, mtay, tay(s),stay(s))
	call meanstd(NC, msfh, sfh(s),ssfh(s))

	! Cross section regression for each year
	fhiy  = sum(simui/simuy,1)/real(T)
	fhsy  = sum(simus/simuy,1)/real(T)
	afhiy(s)=sum(fhiy)/real(nc)
	afhsy(s)=sum(fhsy)/real(nc)


	call drline(nc,fhsy, fhiy, b0, b1, stat1)
	setb(s)  = b1
	stdb(s)  = stat1(7)
		
	! Risk sharing
	grc=log(simuc(2:T,:))-log(simuc(1:T-1,:))
	gry=log(simuy(2:T,:))-log(simuy(1:T-1,:))
	       
    ! Average consumption and GDP across countries at each year
    mc = sum(simuc,2)
	my = sum(simuy,2)
	grmc=log(mc(2:T))-log(mc(1:T-1))
	grmy=log(my(2:T))-log(my(1:T-1))

	do i=1,NC
		pgrc((i-1)*(T-1)+1:i*(T-1))=grc(:,i)
		pgry((i-1)*(T-1)+1:i*(T-1))=gry(:,i)
	    pgrmc((i-1)*(T-1)+1:i*(T-1))=grmc
		pgrmy((i-1)*(T-1)+1:i*(T-1))=grmy
	end do
	
	call drline(nc*(T-1),pgry-pgrmy,pgrc-pgrmc,pdifb(1),pdifb(2),mystat)	
	spdifb(s)=pdifb(2)
	
	!********************
	! correlation between output and investment and savings
	! and  dispersion

	sgdp=100*((simuy(T,:)/simuy(1,:))**(1/real(T-1))-1)
	
	call drline(nc,sgdp,fhiy, b0, b1, stat1)
	csyi(s) = stat1(5)
	dinv(s) = sqrt(stat1(4))
	dgdp(s) = sqrt(stat1(3))
	mgdp(s) = stat1(1)
	
	call drline(nc,sgdp,fhsy, b0, b1, stat1)
	csys(s) = stat1(5)
	dsav(s) = sqrt(stat1(4))
	call  dsvrgn(NC,fhiy,tfhiy)
	call  dsvrgn(NC,fhsy,tfhsy)
	call  dsvrgn(NC,sgdp,tgdp)
	
	l5=floor(NC*0.10)
	u5=floor(NC*0.90)
	
	per(1,1)=sum(tfhiy(1:l5))/real(l5)
	
	per(1,2)=sum(tfhiy(u5:NC))/real(NC-u5+1)
	per(2,1)=sum(tfhsy(1:l5))/real(l5)
	per(2,2)=sum(tfhsy(u5:NC))/real(NC-u5+1)
	per(3,1)=sum(tgdp(1:l5))/real(l5)
	per(3,2)=sum(tgdp(u5:NC))/real(NC-u5+1)

	
end do                

!**********************************************
! output
call meanstd(NSS,spdifb,mpdfib,stdpdifb)
call meanstd(NSS,csys,mcsys,stdcsys)
call meanstd(NSS,csyi,mcsyi,stdcsyi)

write (*,12)
write (*, *)  '******** Enforcement Simulation Report ********'
write (*, 12)
write (*, 24) '  # of simulation:      ', NSS
write (*, 20) '  # of periods:         ', T
write (*, 20) '  # of countries:       ', NC
write (*,12)

write(*, 16)  '  FH coef:              ', sum(setb)/real(NSS),sum(stdb)/real(NSS)
write(*, 16)  '  CA/Y                  ', sum(cay)/real(NSS),sum(scay)/real(NSS)
write(*, 18)  '  TA/Y                  ', sum(tay)/real(NSS),sum(stay)/real(NSS)
write(*, 22)  '  Risk sharing          ', mpdfib,stdpdifb
write(*,16)   '  Time-series S-T corr   ', sum(sfh)/real(NSS),sum(ssfh)/real(NSS)
write(*, *) '-------------------------------' 
write(*,*)  ' Cross-country mean:           '
write(*,14)  '   Investment rate:            ', sum(afhiy)/real(NSS)
write(*,14)  '   Saving rate:                ', sum(afhsy)/real(NSS)
write(*,14)  '   GDP growth rate:            ', sum(mgdp)/real(NSS)
write(*,*)
write(*, *) ' Cross-country correlation:    '
write(*, 14) '   GDP growth and investment rate: ', mcsyi
write(*, 14) '   GDP growth and savings rate:    ', mcsys
write(*,*)   
write(*, *) ' Cross-country dispersion:   '
write(*, 14) '   Std of average investment rate: ', sum(dinv)/real(NSS)
write(*, 14) '   Std of average savings rate:    ', sum(dsav)/real(NSS)
write(*, 14) '   Std of GDP growth rate:        ', sum(dgdp)/real(NSS)
write(*,*)
write(*,*)  '        Investment      Saving       GDP growth'
write(*,902)  'Min 5  ', per(1,1), per(2,1),per(3,1)
write(*,902)  'Max 5  ', per(1,2),per(2,2),per(3,2)
write (*, 12)
write(*, *) '***********************************************'

12 format(/)
14 format(a40,F7.3)
16 format(a40,F7.3,' (',F4.2,')')
18 format(a40,F7.3,' (',F5.2,')')
22 format(a40,F7.3,' (',F5.3,')')
20 format(a40,I2)
24 format(a40,I4)
902 format(A10,f7.3,'       ', f7.3,'       ',f7.3)

END program SimuEnforce
