%**************************************************************************
% This routine compute the FH coefficient, capital flow statistics,
% dispersion of average savings and average investment rate, correlation of
% average savings rate (or investment rate) with the growth rate of real
% GDP per worker.
%
% March 12, 2009  
% Author: Bai and Zhang
%**************************************************************************
% Time period:  1960-2003
% Sample size:  64 countries
% Data Input: 
%     Nominal GDP, nominal final consumption, nominal investment: WDI 2007
%     Final consumption: sum of private consumption and government expenditure
%     Real GDP per capita (Laspeyres): PWT 6.2
%     Population: PWT 6.2
%     Employment: GGDC, missing data are supplemented using PWT 6.2, rgdpc*pop/rgdpwok
%     Capital flow: Lane and Milesi-Ferretti (2007)
% 	   Our net foreign asset position is the sum of net equity asset position(asset-liabilities),
%      net FDI position, net debt position, and net financial derivative position.
%      This is what we called TA in the paper, and TA/Y is the ratio of net foreign asset
%      position over nominal GDP in US dollar.
%      Current account is constructed as the differences of net foreign asset position between two years for each country. 
%
% Output: regression coefficient and capital flow statistics
%
%**************************************************************************

clear all
warning off
%*************************************
% Data loadin and control variables
dirin ='./data/';                % input folder
I=load([dirin,'NGCF.txt']);      % nominal investment
Y=load([dirin,'NGDP.txt']);      % nominal GDP
C=load([dirin,'NCON.txt']);      % nominal final consumption
nY=load([dirin,'RGDP.txt']);     % real GDP per capita
Labor=load([dirin,'Labor.txt']); % employment
POP=load([dirin,'POP.txt']);     % population

cname=importdata([dirin,'countrycode.txt']); % country code
oecd={'AUS','AUT','BEL','CAN','DNK','FIN','GER','GRC','IRL','ITA','JPN',...
    'NLD','NZL','SWE','GBR','USA'};

yearT=1960:2003;
ioecd =0;  % 0: all the countries, 1: oecd countries; 2 non-oecd countries

y1=1960;  % starting year of interest
y2=2003;  % endding year of interest
%*************************************
% Construct saving and investment rate
S=Y-C;    % domestic saving
sy=S./Y;  % savings rate
iy=I./Y;  % investment rate
rgdp=nY.*POP./Labor;  % real GDP per worker

t1=find(yearT==y1);
t2=find(yearT==y2);

% select countries interested
cc=cname;
switch ioecd
    case (0)
        ctext='All Countries';
    case (1)
        ctext='OECD Countries';
        for i=1:length(oecd)
            tt=find(strcmp(oecd(i),cname));
            sy1(:,i)=sy(:,tt);
            iy1(:,i)=iy(:,tt);
            rgdp1(:,i)=rgdp(:,tt);
        end
        sy=sy1;
        iy=iy1;
        rgdp=rgdp1;
        cc=oecd;
    case (2)
        ctext = 'Non-OECD Countries';
        inn=0;
        for i=1:length(cc)
            tt=find(strcmp(cc(i),oecd));
            if (isempty(tt))
                inn = inn+1;
                sy1(:,inn)=sy(:,i);
                iy1(:,inn)=iy(:,i);
                rgdp1(:,inn)=rgdp(:,i);
                cc1(inn)=cc(i);
            end
        end
        sy=sy1;
        iy=iy1;
        cc=cc1;
        rgdp=rgdp1;
end

sy=sy(t1:t2,:);
iy=iy(t1:t2,:);
sgdp=100*((rgdp(t2,:)./rgdp(t1,:)).^(1/(t2-t1+1))-1);
NC=length(cc);

msy=mean(sy,1)';  % average savings rate for each country
miy=mean(iy,1)';  % average investment rate for each country


% FH regression
[b,rsq,e,varb]=ols(msy,miy,1);

% Capital flows
% external position from Lane: 1970-2004
lanename  = load([dirin,'countryCode_lane.txt']);
ebp       = load([dirin,'laneall.txt']);
[tt,wdif] = xlsread([dirin,'namecode_wdi_ifs1.xls']);
ifsf      = load([dirin,'namecode_ifs_wdi.txt']);

nlane=length(lanename);
tlane=length(ebp)/nlane;
eqa=ebp(:,1);
eqb=ebp(:,2);
fdia=ebp(:,3);
fdib=ebp(:,4);
debta=ebp(:,5);
debtb=ebp(:,6);
fda=ebp(:,7);
fdb=ebp(:,8);
exf=ebp(:,end-2);

eqa=reshape(eqa,tlane,nlane);
eqb=reshape(eqb,tlane,nlane);
fdia=reshape(fdia,tlane,nlane);
fdib=reshape(fdib,tlane,nlane);
debta=reshape(debta,tlane,nlane);
debtb=reshape(debtb,tlane,nlane);
fda=reshape(fda,tlane,nlane);
fdb=reshape(fdb,tlane,nlane);


exf=reshape(exf,tlane,nlane);
eb=fdia-fdib+debta-debtb+eqa-eqb+fda-fdb;

gus=ebp(:,end-1);
gus=reshape(gus,tlane,nlane);
eb=eb(1:end-1,:);
gus=gus(1:end-1,:);

ttay=eb./gus;
tcay=(eb(2:end,:)-eb(1:end-1,:))./gus(1:end-1,:);

inan=0;
for i=1:length(cc)
    tt=find(strcmp(cc(i),wdif));
    if (isempty(tt)==0)
        mtt=find(ifsf(tt)==lanename);
        if (isempty(mtt)==0)
            cay1(:,i)=tcay(:,mtt);
            tay(:,i)=ttay(:,mtt);
        else
            cay1(:,i)=nan;
            tay(:,i)=nan;
            inan=inan+1;
        end
    end
end
fprintf(1,'\n');
fprintf(1,'-------- Cross-Country Statistics ---------- \n')
fprintf(1,'%15s (%2.0f), period %4.0f -- %4.0f \n\n',ctext,size(sy,2),y1,y2);
fprintf(1,'  Fh coef = %7.2f (%4.2f)\n',b(2),sqrt(varb(2,2)));
fprintf(1,'  R^2 = %7.2f \n\n',rsq);
fprintf(1,'Capital flows:\n');
fprintf(1,'  CA/Y = %7.2f(%3.2f)  \n',nanmean(nanmean(abs(cay1))),nanstd(nanmean(abs(cay1),1)));
fprintf(1,'  TA/Y = %7.2f(%3.2f)  \n',nanmean(nanmean(abs(tay))),nanstd(nanmean(abs(tay),1)));


% Scatter plot of average savings rate and average investment rate
figure(201)
plot(msy,miy,'*')
title('Cross-Country Saving-Investment')
for i=1:size(sy,2)
    text(msy(i),miy(i),cc(i))
    xlabel('savings rate')
    ylabel('investment rate')
end

% Cross-country statistics for all 64 countries
fprintf(1,'\n');
fprintf(1,'Mean: \n');
fprintf(1,'  Investment rate:          %7.2f\n',mean(miy));
fprintf(1,'  Saving rate:              %7.2f\n',mean(msy));
fprintf(1,'  GDP growth rate(percent): %7.2f\n\n',mean(sgdp));


fprintf(1,'Correlation: \n');
tmp=corrcoef(msy,miy);
sssi=tmp(1,2);
fprintf(1,'  Savings and investment rate:         %7.2f\n',tmp(1,2));
tmp=corrcoef(sgdp,miy);
ssyi=tmp(1,2);
fprintf(1,'  GDP growth rate and investment rate: %7.2f\n',tmp(1,2));
tmp=corrcoef(sgdp,msy);
ssys=tmp(1,2);
fprintf(1,'  GDP growth rate and saving rate:     %7.2f\n\n',tmp(1,2));

% dispersion of gpd and saving
dsav=std(msy);
dinv=std(miy);
dgdp=std(sgdp);
fprintf(1,'Dispersion: \n');
fprintf(1,'  Std of average investment rate: %7.2f\n',dinv);
fprintf(1,'  Std of average saving rate:     %7.2f\n',dsav);
fprintf(1,'  Std of GDP growth rate:         %7.2f\n\n',dgdp);

% lower and higher 10% percentile
l5=floor(NC*0.10);
u5=floor(NC*0.90);

tsav=sort(msy);
sl5 =mean(tsav(1:l5));
su5 =mean(tsav(u5:end));

tinv=sort(miy);
il5 =mean(tinv(1:l5));
iu5 =mean(tinv(u5:end));

tgdp=sort(sgdp);
gl5 =mean(tgdp(1:l5));
gu5 =mean(tgdp(u5:end));

fprintf(1,'                               Investment      Saving       GDP growth \n');
fprintf(1,'Average of Min 10 percentile:   %7.2f       %7.2f        %7.2f\n',il5,sl5,gl5);
fprintf(1,'Average of Max 10 percentile:   %7.2f       %7.2f        %7.2f\n',iu5,su5,gu5);


 

