%**************************************************************************
% Discretizing regime-switching process
%
% Method: Tauchen method
%    y(t)=z(t)'beta(s(t))+u(t), u(st)~ i.i.d. N(0, sigma(st)^2)
%    z(t): expaining variables including lagged values of y
%    Ne: number of explaining variables
%    regime s(t), an unobserved N-state Markov chain.
% 
% Reference: Tauchen (1986), Economics Letters 20, 177-181
% Author: Bai and Zhang
% March 12, 2009
%**************************************************************************

clear all;
warning off

% parameters
N   = 3;                               % Number of regimes

% Control variables
ND          = [3 3 3];                 % number of discrete points, same for each AR(1) processs;
iscale      = 3;                       % range of points around conditional means
wfolder     = 'shock9.dat';            % writeout folder

%********************************************************************************************
% Load in parameters 
regime3=load('newregime.inp');
for i = 1:N,
    tbeta(1,i)   = regime3((i-1)*3+1);       % mu in each regime
    tbeta(2,i)   = regime3((i-1)*3+2);       % rou in each regime
    tsigma(1,i)  = regime3((i-1)*3+3);       % sigma in each regime
end
for i = 1:N,
    for j = 1:N,
        tp(i,j)  = regime3(N*3 + (i-1)*N + j);
    end
end

%**************************************************************************
% sort regimes in the mean
[ssigma(1,:), isigma] = sort(tbeta(1,:),'ascend');
beta = tbeta(:,isigma);
sigma = tsigma(isigma);

for i=1:N,
    p(i,:)=tp(i,isigma);
end
tp = p;
for i=1:N,
    p(:,i)=tp(isigma,i);
end


%**************************************************************************
% Discretizing points chosen
ys =3.38;
ye =4.48;
ym=(ys+ye)/2;

y  = ys : (ye-ys)/(sum(ND)-1) : ye;
nn=ND(1)+ND(2)+ND(3);
for i=1:ND(1)
    y(i)=ys+(i-1)*sigma(1)*iscale;
end
for i=1:ND(3)
    y(nn-i+1)=ye-(i-1)*sigma(3)*iscale;
end

y(5)=ym;
y(4)=ym-sigma(2)*iscale;
y(6)=ym+sigma(2)*iscale;

dis = y(2:end)-y(1:end-1);
w   = dis/2;

NN=sum(ND);
cumND=cumsum(ND);
cumND=[0 cumND];

% Construct transition matrix
for i=1:N                             % starting regime i
    for j=1:ND(i)                     % each point in regime i
        for ii=1:N                    % endding regime ii
            for jj=1:ND(ii)           % each point in regime ii
                is = cumND(i)+j ;
                ie = cumND(ii)+jj;
                if jj==1,
                    pai(is,ie)=p(i,ii)*normp((y(ie)+w(ie)-beta(1,ii)*(1-beta(2,ii))-beta(2,ii)*y(is))/sigma(ii));
                elseif jj==ND(ii),
                    pai(is,ie)=p(i,ii)*(1-normp((y(ie)-w(ie-1)-beta(1,ii)*(1-beta(2,ii))-beta(2,ii)*y(is))/sigma(ii)));
                else
                    pai(is,ie)=p(i,ii)*(normp((y(ie)+w(ie)-beta(1,ii)*(1-beta(2,ii))-beta(2,ii)*y(is))/sigma(ii))-...
                            normp((y(ie)-w(ie-1)-beta(1,ii)*(1-beta(2,ii))-beta(2,ii)*y(is))/sigma(ii)));
                end           
            end                       % end of iteration over jj
        end                           % end of iteration over ii
    end                               % end of iteration over j
end                                   % end of iteration over i


% clean the process
tpp = round(pai*1.0e4)/1.0e4;
for i=1:NN
    [mt,imt] = max(tpp(i,:));
    mtmp = 1.0-(sum(tpp(i,:))-tpp(i,imt));
    tpp(i,imt) = mtmp;
end
pai = tpp;

%---------------------
% Calculate statistics for discretized process
pp=pai^100000000;
invpai=pp(1,:);
inve=sum(invpai.*y);
invsee=0.0;
for i=1:length(y)
    invsee=invsee+(y(i)-inve)^2*invpai(i);
end
invsee=invsee^0.5;

ss=0.0;
for i=1:length(y)
    for j=1:length(y)
        ss=ss+y(i)*y(j)*invpai(i)*pai(i,j);
    end
end

fprintf(1,'Discretize     mean        std       cv         corr \n');
fprintf(1,'              %7.4f    %7.4f   %7.4f    %7.4f\n',inve,invsee,invsee/inve,(ss-inve^2)/(invsee^2));


% write shock
y     = y-(y(1)-2);
y     = exp(y)./50;
fid   = fopen(wfolder,'w');
for i = 1:NN,
    fprintf(fid,'%20.6e \n',y(i));
end
for i = 1:NN,
    for j = 1:NN,
        fprintf(fid,'%20.6e \n',pai(i,j));
    end
end
fclose(fid);

