%**************************************************************************
% Compute productivity process from the data
% March 20, 2009
% Author: Bai and Zhang
%**************************************************************************
% Sample size: 64 countries
% Time period: 1960-2003
%
% National account data are from PWT 6.2 (real GDP per capita (Laspeyres)
% and investment share (ki))
%
% Employment data: from GGDC, missing data are supplemented from PWT 6.2
% (*) PWT 62 constructed from GDP per capita and GDP per worker
%
% Output: total productivity process
%
%**************************************************************************
%
% Note:
% To calibrate A process, we need real GDP, capital stock (constructed from real investment flows) and labor.
%       A=y/(l^(1-alpha) * k^alpha)
%***************************************************************************
clear;
close all
warning off

%**********************************************************************************
% Parameters
delta      = 0.10;       % capital depreciation rate
alpha      = 0.33;       % share of capital

iys        = 1960;       % starting year of interest
iye        = 2003;       % endding year of interest

draww1     = 0;          % 1: draw detrened productivity process in one graph
draww2     = 0;          % 1: plot detrened productivity process for each country in separate graphs

folder1    = './data/';
folderout  = './';

%**********************************************************************************                 
% Input
LABOR = load([folder1,'Labor.txt']);     % Employment
RGCF  = load([folder1,'RGCF.txt']);      % Real investment per capita
RGDP  = load([folder1,'RGDP.txt']);      % real GDP per capita
POP   = load([folder1,'POP.txt']);       % population

RGDP       = RGDP.*POP;
RGCF       = RGCF.*POP;
Allname    = importdata([folder1,'countryCode.txt']); % 64 country codes
oecd               = importdata([folder1,'oecd.txt']); % country codes for 16 OECD countries

yearY      = 1960:2003;
        
%**************************************************************************
% Construct Capital
sizeY = size(RGDP,1);
gy    = (RGDP(end,:)./RGDP(1,:)).^(1/(sizeY-1));   % Average growth rate of GDP for each country
invr  = RGCF./RGDP;                                % Investment ratio over time
ainv  = mean(invr,1);                              % Average investment rate
capy  = ainv./(gy-1+delta);                        % K/Y ratio in the balanced growth path
k1    = capy.*RGDP(1,:);                           % Initial capital stock in 1960 or ys

CAPITAL=[k1];
for i=2:sizeY,
    ktemp=CAPITAL(end,:)*(1-delta)+RGCF(i-1,:);
    CAPITAL=[CAPITAL; ktemp];
end

%**************************************************************************
% Construct productivity shock

TAshock  = 1/10*RGDP./(CAPITAL.^alpha.*LABOR.^(1-alpha));        % productivity shock 

%**************************************************************************
countries  = Allname;     

% Year chosen
ys     = find(yearY==iys);
ye     = find(yearY==iye);
yearY  = yearY(ys:ye);
sizeY  = length(yearY);     % number of years
sizeC  = length(countries);  % number of countries
%**************************************************************************
% Detrend with common growth rate
Ashock       = log(TAshock);
tmp          = (Ashock(end,:)-Ashock(1,:))./(sizeY-1);
detr         = mean(tmp);
tmp          = ones(sizeY,1)*detr;
ctmp         = cumsum(tmp)-detr;
cttmp        = ctmp*ones(1,sizeC);
dAshock      = Ashock-cttmp;

%**************************************************************************
% Write out the output
% Future input for Fortran code to compute random process
fid=fopen([folderout,'/dAshock09.dat'],'w');
fid2=fopen([folderout,'/default09.dat'],'w'); % a file with all zeros

for i=1:sizeC,
    for j=1:sizeY,
        fprintf(fid,'%2.4f\n',dAshock(j,i));
        fprintf(fid2,'%4.0f\n',0);
    end
end
fclose(fid);
fclose(fid2);

%****************************************************************************
% Plot detrended productivity process
if draww1==1,
    figure(22)
    for i=1:sizeC,
        plot(yearY,dAshock(:,i),'Color',[0.5 0.5 0.5],'LineWidth',0.25);
        char=['Productivity Processes (',num2str(sizeC),' )'];
        title(char,'FontSize', 16, 'FontWeight', 'bold')
        xlim([yearY(1),yearY(end)]);
        hold on
    end
end

if draww2==1,
    for i=1:64
        figure(260+i-1),
        plot(yearY,dAshock(:,i),'r');
        aa=strcat('A process for ',countries(i));
        ylim([2.5, 5.0]);
        title(aa);
    end
end

%**************************************************************************
%------ TFP Statistics --------

%**************************************************************************
ms = mean(mean(dAshock));
for i=1:size(dAshock,1)
    mstd1(i)=std(dAshock(i,:));
end
mstd=mean(mstd1);
mcv=mean(mstd1'./mean(dAshock,2));
for i=1:size(dAshock,2)
    cor=corrcoef(dAshock(2:end,i),dAshock(1:end-1,i));
    mcor1(i)=cor(1,2);
end
mcor=mean(mcor1);

inn=0;
for i=1:length(oecd)
    tt=find(strcmp(oecd(i),countries));
    if (isempty(tt)==0)
        inn=inn+1;
        rdAshock(:,inn)=dAshock(:,tt);
    end
end
inn=0;
for i=1:length(countries)
    tt=find(strcmp(oecd,countries(i)));
    if (isempty(tt)==1)
        inn=inn+1;
        pdAshock(:,inn)=dAshock(:,i);
    end
end

fprintf(1,'\n');
fprintf(1,'------- TFP Statistics -------- \n');
fprintf(1,' mean        std       cv         corr \n');
fprintf(1,'%7.4f    %7.4f   %7.4f    %7.4f\n',ms,mstd,mcv,mcor);
fprintf(1,'\n');
fprintf(1,'Time-series coefficient of variation for TFP\n');
fprintf(1, '  All  countries: %7.4f \n',mean(std(dAshock))/mean(mean(dAshock)));
fprintf(1, '  Oecd countries: %7.4f \n',mean(std(rdAshock))/mean(mean(rdAshock)));
fprintf(1, '  Poor countries: %7.4f \n',mean(std(pdAshock))/mean(mean(pdAshock)));

ncc=size(dAshock,2);
n10=floor(ncc*0.05);
n90=floor(ncc*0.95);

for i=1:size(dAshock,1)
    ta=sort(dAshock(i,:));
    ann(i,1)=ta(n10);
    ann(i,2)=ta(n90);
end

%plot for paper
% control for figures
lwidth  = 2.0;
lmc     = 20;
smc     = 18;
recc    = [0.42, 0.80, 9.8, 6.5];

pickr={'USA';'DNK';'FIN'};
pickp={'SYR';'ZWE';'IDN'};
%picks={'URY';'SLV';'IRN';'PER';'CMR'};
picks={'IRN';'SLV';'PER'};


ccolor=['r';'b';'k';'r';'b';'k';'m'];
cmark=['- ';'-*';'-o';'-d';'--';'-p';'->';'-p'];
lwd=[2;2;2.0;2.0;2.5;2.0];

figure(25)
hold on
for i=1:length(pickr)
    tt=find(strcmp(countries,pickr(i)));
    plot(1960:2003,dAshock(:,tt),cmark(i,:),'LineWidth',lwd(i),'Color',ccolor(i));
    ylim([2.5,5])
    xlim([1960,2003])
    text(1980,dAshock(25,tt),char(pickr(i)),'Color',ccolor(i),'FontSize',lmc);
end

for i=1:length(pickp)
    tt=find(strcmp(countries,pickp(i)));
    plot(1960:2003,dAshock(:,tt),cmark(i+length(pickr),:),'LineWidth',lwd(i+length(pickr)),'Color',ccolor(i));
    ylim([2.5,5])
    xlim([1960,2003])
    text(1980,dAshock(25,tt),char(pickp(i)),'Color',ccolor(i),'FontSize',lmc);
end

set(gca,'FontSize',lmc)
axis square
set(gca,'Box','on')
set(gcf,'PaperPositionMode','auto')
set(gcf,'PaperPosition',recc);
set(gcf,'PaperOrientation','landscape')

cmark=['- ';'-*';'--';'-d';'--';'-p';'->';'-p'];
lwd=[2;2;2.0;2.0;2.5;2.0];

figure(26)
hold on
for i=1:length(picks)   
    tt=find(strcmp(countries,picks(i)));
    plot(1960:2003,dAshock(:,tt),cmark(i,:),'LineWidth',lwd(i),'Color',ccolor(i));
    ylim([2.5,5])
    xlim([1960,2003])
    text(1980,dAshock(25,tt),char(picks(i)),'Color',ccolor(i),'FontSize',lmc);
end
set(gca,'FontSize',lmc)
axis square
set(gca,'Box','on')
set(gcf,'PaperPositionMode','auto')
set(gcf,'PaperPosition',recc);
set(gcf,'PaperOrientation','landscape')
