%**************************************************************************
% Business Cycle Statistcs and Risk Sharing Coef for 64 countries
% 
% Author Bai and Zhang
% March 12, 2009 
%**************************************************************************
% Time period:  1960-2003
% Sample size:  64 countries
% Data source: Penn World Table 6.2
%     real GDP per capita: RGDP.txt
%     real final consumption per capita: constructed using rgdpl*(kc+kg)/100
%     real investment per capita: constructed using rgdpl*ki/100
%     net export per capita: constructed using rgdpl*(100-kc-kg-ki)/100
%     population
%  
% Output: business cycle statistics. 
%     We first times all the per capita data with population. We then 
% take log and HP-filer of all the data series except for savings and 
% net export, as these two may contain negative entries. Finally, we 
% calculate the business-cycle statistics and compute international risk sharing. 
%
%**************************************************************************
clear all
warning off

%********************************************
% Control variables
dirin = './data/';            % input folder
nx=load([dirin,'RNEX.txt']);  % nex export per capita
con=load([dirin,'RCON.txt']); % final consumption per capita
gdp=load([dirin,'RGDP.txt']); % real gdp per capita
inv=load([dirin,'RGCF.txt']); % real investment per capita
pop=load([dirin,'POP.txt']);  % population
cname=importdata([dirin,'countryCode.txt']); % 64 country codes
%*****************************************
T=size(gdp,1);                 % number of periods
NC=size(gdp,2);                % number of countries

% original variables
sav=(gdp-con).*pop;            % domestic savings
ncon = con;
ngdp = gdp;
ninv=inv.*pop;

% logged variables
nxy=nx./gdp;
con=log(con.*pop);
gdp=log(gdp.*pop);
inv=log(inv.*pop);

w = 100;
stdn = nanstd(nxy,1);
for i=1:NC
    hgdp=hpfilter(gdp(:,i),w);
    stdy(i)=std(hgdp);
	
    % consumption standard deviation and correlation with y
    ttt=find(isnan(con(:,i))==1);
    if (isempty(ttt)==1)
        id=1;
    else
        id=ttt(end)+1;
    end
    if (id<T)
        zcon=con(id:end,i);
        zgdp=hgdp(id:end);
        hcon=hpfilter(zcon,w);
        stdc(i)=std(hcon)/stdy(i);
        tmp=corrcoef(hcon,zgdp);
        ccy(i)=tmp(1,2);
    else
        stdc(i)=nan;
        ccy(i)=nan;
    end
    
    % investment standard deviation and correlation with y and s
    ttt=find(isnan(inv(:,i))==1);
    if (isempty(ttt)==1)
        id=1;
    else
        id=ttt(end)+1;
    end
    if (id<T)
        zinv=inv(id:end,i);
        zgdp=hgdp(id:end);
        zsav=sav(id:end,i);
        nzinv=ninv(id:end,i);
        hinv=hpfilter(zinv,w);
        stdi(i)=std(hinv)/stdy(i);
        tmp=corrcoef(hinv,zgdp);
        ciy(i)=tmp(1,2);
        gsav=zsav(2:end)./zsav(1:end-1)-1;
        ginv=zinv(2:end)./zinv(1:end-1)-1;
        tmp=corrcoef(gsav,ginv);
        csi(i)=tmp(1,2);
    else
        stdi(i)=nan;
        ciy(i)=nan;
        csy(i)=nan;
    end
    
    % net export correlation with y
    ttt=find(isnan(nxy(:,i))==1);
    if (isempty(ttt)==1)
        id=1;
    else
        id=ttt(end)+1;
    end
    if (id<T)
        znxy=nxy(id:end,i);
        zgdp=hgdp(id:end);
        tmp=corrcoef(znxy,zgdp);
        cny(i)=tmp(1,2);
    else
        cny(i)=nan;
    end
    
end

% Cross-country correlation
cyy=[];
ccc=[];
for i=1:NC-1
    for j=i+1:NC
        hgdp1=hpfilter(gdp(:,i),w);
        hgdp2=hpfilter(gdp(:,j),w);
        tmp=corrcoef(hgdp1,hgdp2);
        cyy=[cyy;tmp(1,2)];
        
        ttt1=find(isnan(con(:,i))==1);
        ttt2=find(isnan(con(:,j))==1);
        if (isempty(ttt1)==1)
            id1=1;
        else
            id1=ttt1(end)+1;
        end
        if (isempty(ttt2)==1)
            id2=1;
        else
            id2=ttt2(end)+1;
        end
        id=max([id1,id2]);
        if (id<T)
            hcon1=hpfilter(con(id:end,i),w);
            hcon2=hpfilter(con(id:end,j),w);
            tmp=corrcoef(hcon1,hcon2);
            ccc=[ccc;tmp(1,2)];
        else
            ccc=nan;
        end

    end
end
fprintf(1,'Standard Deviation  \n');
fprintf(1,'\t\t GDP: %7.2f(%3.2f) \n ',nanmean(stdy)*100,nanstd(stdy)*100);
fprintf(1,'\t\t Net export/GDP: %7.2f(%3.2f) \n',nanmean(stdn)*100,nanstd(stdn)*100);
fprintf(1,'Relative Standard Deviation \n');
fprintf(1,'\t\t Consumption %7.2f(%3.2f) \n',nanmean(stdc),nanstd(stdc));
fprintf(1,'\t\t Investment %7.2f(%3.2f) \n',nanmean(stdi),nanstd(stdi));
fprintf(1,'Correlation with GDP \n');
fprintf(1,'\t\t Consumption %7.2f(%3.2f) \n',nanmean(ccy),nanstd(ccy));
fprintf(1,'\t\t Investment %7.2f(%3.2f) \n',nanmean(ciy),nanstd(ciy));
fprintf(1,'\t\t Net Export/GDP %7.2f(%3.2f) \n',nanmean(cny),nanstd(cny));
fprintf(1,'Cross-Country Correlation \n');
fprintf(1,'\t\t GDP %7.2f(%3.2f) \n',nanmean(cyy),nanstd(cyy));
fprintf(1,'\t\t Consumption %7.2f(%3.2f) \n',nanmean(ccc),nanstd(ccc));
fprintf(1,'Time-series saving-investmet Correlation %7.2f(%3.2f)\n',nanmedian(csi),nanstd(csi));


% International Risk Sharing
lcon=log(ncon(2:end,:))-log(ncon(1:end-1,:));
lgdp=log(ngdp(2:end,:))-log(ngdp(1:end-1,:));
acon=nanmean(ncon,2);
agdp=nanmean(ngdp,2);
lacon=log(acon(2:end))-log(acon(1:end-1));
lagdp=log(agdp(2:end))-log(agdp(1:end-1));

tt=size(lcon,1);
nn=size(lcon,2);
yvar=reshape(lcon-kron(lacon,ones(1,nn)),tt*nn,1);
xvar=reshape(lgdp-kron(lagdp,ones(1,nn)),tt*nn,1);
mm=find(isnan(yvar));
yvar(mm)=[];
xvar(mm)=[];
[rs,rsq,e,varb]=ols(xvar,yvar,1);
fprintf(1,'Risksharing coef: %7.2f(%3.2f)\n',rs(2),sqrt(varb(2,2)));

    
    
  